import pandas as pd
"""Equity Module"""
class Equity:
    def __init__(self, session):
        self.session = session

    """
    Get Info for a single company
    """
    def info(self, symbol):
        url = f"https://www.nseindia.com/api/quote-equity?symbol={symbol}"
        response = self.session.get(url)
        #df = pd.DataFrame(response.json())
        #return df
        return response.json()
    
    """
    Get History for a single company
    Inputs:
    symbol: str: Symbol of the company
    from_date: str: Start Date in dd-mm-yyyy format
    to_date: str: End Date in dd-mm-yyyy format
    """
    def history(self, symbol, from_date, to_date):
        url = f"https://www.nseindia.com/api/historical/cm/equity?symbol={symbol}"
        params = dict()
        params['from'] = from_date
        params['to'] = to_date

        data = self.session.get(url, params=params).json()

        # Extract relevant columns
        columns = [
            "CH_TIMESTAMP", "CH_TRADE_HIGH_PRICE", "CH_TRADE_LOW_PRICE", "CH_OPENING_PRICE",
            "CH_CLOSING_PRICE", "CH_LAST_TRADED_PRICE", "CH_PREVIOUS_CLS_PRICE",
            "CH_TOT_TRADED_QTY", "CH_TOT_TRADED_VAL", "CH_52WEEK_HIGH_PRICE",
            "CH_52WEEK_LOW_PRICE", "CH_TOTAL_TRADES", "VWAP"
        ]

        # Create DataFrame
        df = pd.DataFrame([{col: entry.get(col, None) for col in columns} for entry in data["data"]])
        return df
    
    """Get Market Status"""
    def market_status(self):
        res = self.session.get("https://www.nseindia.com/api/marketStatus")
        return res.json()


class CommodityOptions:
    def __init__(self, session):
        self.session = session

    """ CAUTION: Not in use"""
    def commodity_options_list(self):
        url = "https://www.nseindia.com/api/quotes-commodity-derivatives-master"
        #df = pd.DataFrame(self.session.get(url).json())
        #df.to_csv('h.csv')
        return self.session.get(url).json()


import pandas as pd
"""Insider Class"""
class Insider:
    def __init__(self, session):
        self.session = session

    def insider_data(self, symbol=None, from_date=None, to_date=None):
        """
        Get Insider Data for a single company or all companies.
        
        Parameters:
        - symbol (str, optional): Symbol of the company. If None, fetches data for all companies.
        - from_date (str, optional): Start Date in dd-mm-yyyy format.
        - to_date (str, optional): End Date in dd-mm-yyyy format.

        Returns:
        - dict or list: JSON response containing insider data.
        """
        
        url = "https://www.nseindia.com/api/corporates-pit?index=equities"
        
        # Append symbol parameter if filtering by a single company
        if symbol:
            url += f"&symbol={symbol}"
        
        params = {}
        if from_date:
            params["from_date"] = from_date
        if to_date:
            params["to_date"] = to_date

        try:
            response = self.session.get(url, params=params)
            response.raise_for_status()  # Raise an error for HTTP errors (4xx, 5xx)
            data = response.json()
            
            # Make df out of 'data' key directly if present, else return full response
            df = pd.DataFrame(data.get("data", data))
            
            return df
        
        except Exception as e:
            return {"error": str(e)}

    
    """
    Get Pledged Data for a single company
    """
    def getPledgedData(self, symbol):
        url = f"https://www.nseindia.com/api/corporate-pledgedata?index=equities&symbol={symbol}"
        response = self.session.get(url)
        return response.json()
    
    """
    Get Sast Data for a single company
    """
    def getSastData(self, symbol, from_date=None, to_date=None):
        url = "https://www.nseindia.com/api/corporate-sast-reg29"
        params = dict()
        params["index"] = 'equities'
        params["symbol"] = symbol
        if from_date and to_date:
            params["from_date"] = from_date
            params["to_date"] = to_date

        response = self.session.get(url, params=params)
        df = pd.DataFrame(response.json()['data'])
        return df


import pandas as pd
from .utils import extract_option_data

class EquityOptions:
    def __init__(self, session):
        self.session = session

    def fno_stocks_list(self):
        # Returns a list of FNO stocks
        url = "https://www.nseindia.com/api/master-quote"
        return self.session.get(url).json()
    
    def expiry_dates(self, symbol):
        # Returns a list of expiry dates for a given symbol
        url = f"https://www.nseindia.com/api/option-chain-equities?symbol={symbol}"
        return self.session.get(url).json()['records']['expiryDates']
    
    def option_chain(self, symbol, expiry_date=None, strike_price=None):
        """
        Filters options data for a specific expiry date and/or strike price.

        Args:
            symbol (str): The symbol of the index.
            expiry_date (str, optional): The expiry date to filter by. Defaults to None.
            strike_price (float, optional): The strike price to filter by. Defaults to None.

        Returns:
            list: A list of options data entries that match the filter criteria.
        """
        url = f"https://www.nseindia.com/api/option-chain-equities?symbol={symbol}"
        records = self.session.get(url).json()['records']
        data = records['data']

        filtered_data = data

        if expiry_date:
            filtered_data = [entry for entry in filtered_data if entry.get('expiryDate') == expiry_date]

        if strike_price is not None:  # Explicitly check for None
            filtered_data = [entry for entry in filtered_data if entry.get('strikePrice') == strike_price]

        df = pd.DataFrame(filtered_data)

        # Extract data for both PE and CE
        pe_df = extract_option_data(df["PE"])
        ce_df = extract_option_data(df["CE"])

        # Rename columns to avoid confusion
        pe_df.columns = [f"PE_{col}" for col in pe_df.columns]
        ce_df.columns = [f"CE_{col}" for col in ce_df.columns]

        # Combine with the original DataFrame
        df_final = pd.concat([df, pe_df, ce_df], axis=1)

        # Drop the original PE and CE columns (optional)
        df_final = df_final.drop(columns=["PE", "CE"])

        # Expiry Dates
        expiry_dates = records['expiryDates']

        # Underlying Value
        underlying_value = records['underlyingValue']
        # Return the final DataFrame
        return df_final, expiry_dates, underlying_value



"""
NSEPY - NSE Python API

Written by Anil Sardiwal
"""
import pandas as pd

from nsesession import NSESession
from equity.equity import Equity
from equity.insider import Insider
from derivatives.indicesOptions import IndicesOptions
from derivatives.equityOptions import EquityOptions
from derivatives.commodityOptions import CommodityOptions
from mf.mf import MF

class NSE:
    def __init__(self):
        self.session = NSESession()

        self.equity = Equity(self.session)
        self.insider = Insider(self.session)
        self.options = IndicesOptions(self.session)
        self.equityOptions = EquityOptions(self.session)
        self.commodity_options = CommodityOptions(self.session)
        self.mf = MF(self.session)

    """
    TODO Fix or Move to NSESession
    Get NSESession object data
    """
    def session_info(self):
        """
        Returns a dictionary containing all attributes of the self.session object.
        """
        session_data = {}
        for attr_name in dir(self.session):
            # Avoid including special methods (starting with __) and attributes that might cause errors
            if not attr_name.startswith("__") and not callable(getattr(self.session, attr_name)):
                try:
                    session_data[attr_name] = getattr(self.session, attr_name)
                except Exception as e:
                    print(f"Warning: Could not access attribute '{attr_name}' due to error: {e}")
                    # Optionally handle the error differently, e.g., log it or skip the attribute

        return session_data

    """
    AutoCompletes the query and returns in JSON format
    """
    def autocomplete(self, query):
        url = f"https://www.nseindia.com/api/search/autocomplete?q={query}"
        return self.session.get(url).json()

    """Endpoint Tester"""
    def endpoint_tester(self, endpoint_url):
        res = self.session.get(endpoint_url)
        return res

if __name__ == "__main__":
    nse = NSE()

    """Endpoint Tester"""
    data = nse.endpoint_tester("https://www.nseindia.com/api/option-chain-indices?symbol=NIFTY").json()['records']['data']
    df = (pd.DataFrame(data))
    df.to_csv("h.csv")
    

    """Autocomplete"""
    #print(nse.autocomplete("Info"))


"""
Just a test usage file. Not to be used in production.
"""
import pandas as pd
from nse import NSE

if __name__ == "__main__":
  nse = NSE()
  """Session Data"""
  #print(nse.session_info())

  """Endpoint Tester"""
  #print(nse.endpoint_tester("https://www.nseindia.com/api/historical/foCPV?from=01-02-2025&to=15-02-2025&instrumentType=OPTIDX&symbol=NIFTY&year=2025&expiryDate=20-Feb-2025&optionType=CE").json())
  #print(nse.endpoint_tester("https://www.nseindia.com/api/option-chain-indices?symbol=NIFTY").json())

  """Autocomplete"""
  #print(nse.autocomplete("info"))

  # Equity Examples
  """INFO"""
  #print(nse.equity.info("SBIN")) #Info
  #print(nse.equity.info("SBIN")['priceInfo']['lastPrice']) # CMP
  #print(nse.equity.info("SBIN")['info']) #Info

  """History"""
  #print(nse.equity.history("ESCORTS", "01-02-2025", "14-02-2025"))

  """Market Status"""
  #print(nse.equity.market_status())

  """
  Insider Data
  """
  #print(nse.insider.insider_data())    #All Insider Data
  #print(nse.insider.insider_data(from_date="11-12-2023", to_date="14-02-2025"))  #All Insider Data with date range
  #print(nse.insider.insider_data("SBIN"))  # Insider Data for a single company
  #print(nse.insider.insider_data("INFY", from_date="11-12-2023", to_date="14-02-2025")) # Insider Data for a single company with date range

  """
  Pledged Data
  """
  #print(nse.insider.getPledgedData("ESCORTS"))  # Pledged Data for a single company

  """
  Sast Data
  """
  #print(nse.insider.getSastData("ESCORTS"))
  #print(nse.insider.getSastData("INFY", from_date="01-01-2024", to_date="01-02-2025"))

  """MF Insider Data"""
  #print(nse.mf.mf_insider_data(from_date="01-02-2025", to_date="02-02-2025"))
  #print(nse.mf.mf_insider_data(isin="INF879O01027"))
  #print(nse.mf.mf_insider_data(symbol="PPFAS Mutual Fund"))

  """ Indices Options"""
  #print(nse.options.option_chain("NIFTY")[0]) #Option Chain - all data
  #print(nse.options.option_chain("NIFTY", expiry_date="06-Mar-2025")[0]) #Option Chain - by expiry date
  #print(nse.options.option_chain("NIFTY", strike_price=22000)[0]) #Option Chain - by strike price
  #print(nse.options.option_chain("NIFTY", expiry_date="06-Mar-2025", strike_price=22000)[0]) #Option Chain - by expiry date & strike price
  #print(nse.options.option_chain("NIFTY")[1]) #Option Chain - All Expiries
  #print(nse.options.option_chain("NIFTY")[2]) #Option Chain - Underlying Value
  #print(nse.options.expiry_dates("NIFTY")) #Expiry Dates
  #option_chain, expiry_dates, underlying_value = nse.options.option_chain("NIFTY")
  #print(nse.options.get_indices()) #Indices List
   
  """ Equity Options """
  #print(nse.equityOptions.fno_stocks_list()) #FNO Stocks List
  #print(nse.equityOptions.option_chain("SBIN")[0]) #Option Chain - all data
  #print(nse.equityOptions.option_chain("SBIN", expiry_date="27-Mar-2025")[0]) #Option Chain - by expiry date
  #print(nse.equityOptions.option_chain("SBIN", strike_price=800)[0]) #Option Chain - by strike price
  #print(nse.equityOptions.option_chain("SBIN", expiry_date="27-Mar-2025", strike_price=800)[0]) #Option Chain - by expiry date & strike price
  #print(nse.equityOptions.option_chain("SBIN")[1]) #Option Chain - All Expiries
  #print(nse.equityOptions.option_chain("SBIN")[2]) #Option Chain - Underlying Value
  #print(nse.equityOptions.expiry_dates("SBIN")) #Expiry Dates
  #option_chain, expiry_dates, underlying_value = nse.equityOptions.option_chain("SBIN")

  """ Commodity Options """
  #print(nse.commodity_options.commodity_options_list()) # Not in usage


import pandas as pd
class MF:
    def __init__(self, session):
        self.session = session

    """
    Event Based Disclosure of Mutual Funds
    Tells which employees of the MF have bought or sold the shares of the MF

    URL = https://www.nseindia.com/companies-listing/corporate-filings-EBD

    Parameters:
    symbol: str - Symbol of the Mutual Fund
    from_date: str - Start date of the data to be fetched
    to_date: str - End date of the data to be fetched
    isin: str - ISIN of the Mutual Fund
    """ 
    def mf_insider_data(self, symbol=None, from_date=None, to_date=None, isin=None):
        url = f"https://www.nseindia.com/api/corporate-event-disclosure?index=Ebddata"
        if from_date and to_date:
            url += f"&from_date={from_date}&to_date={to_date}"
        if isin:
            url += f"&isin={isin}"
        if symbol:
            url += f"&symbol={symbol}"

        data = self.session.get(url).json()
        df = pd.DataFrame(data)
        return df


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="pnsea",
    version="0.2.22",
    author="Anil Sardiwal",
    author_email="theonlyanil@gmail.com",
    license_files = ('LICENSE'),
    description="A Python library for accessing data from the National Stock Exchange (NSE) of India.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/theonlyanil/nsepy",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        "stealthkit",
        "pandas",
    ],
)


import pandas as pd

# Define the required fields to extract for both CE and PE
required_fields = [
    "openInterest",
    "changeinOpenInterest",
    "impliedVolatility",
    "lastPrice",
    "totalTradedVolume",
    "bidQty",
    "bidprice",
    "askQty",
    "askPrice",
]

# Function to extract required fields from a dictionary column
def extract_option_data(option_series):
    extracted_data = []
    for option_dict in option_series:
        if isinstance(option_dict, dict):
            extracted_data.append({key: option_dict.get(key, None) for key in required_fields})
        else:
            extracted_data.append({key: None for key in required_fields})
    return pd.DataFrame(extracted_data)